/****************************************************************************\
  FILE..........: tl30p_bc.cpp
  COPYRIGHT.....: (c) 1993-1995, Bennet-Tec Information Systems, Inc
  VERSION.......: 3.99
  DESCRIPTION...: TVbxList class implementation
\****************************************************************************/

#include <owl\owlpch.h>
#pragma hdrstop

#include <owl\vbxctl.h>
#include "tl30p_bc.h"

/////////////////////////////////////////////////////////////////////////////
class TListLoader
{
  private:
            HMODULE           m_hmod;
            FARPROC           m_fpFreeBuffer;
            FARPROC           m_fpIsValidBuffer;
            FARPROC           m_fpTranslateIndex;
            FARPROC           m_fpGetItemByXY;
            FARPROC           m_fpGetItemRect;
            FARPROC           m_fpSaveBuffer;
            FARPROC           m_fpLoadBuffer;
            FARPROC           m_fpCopyBuffer;
            FARPROC           m_fpPasteBuffer;
            FARPROC           m_fpIsClipboardFormatAvailable;
            FARPROC           m_fpIndexByBM;
            FARPROC           m_fpIsValidBM;
            FARPROC           m_fpFindItem;

        protected:
  public:
                              TListLoader       ();
                             ~TListLoader       ();

            void              FreeBuffer        (HTREEBUFFER);
            BOOL              IsValidBuffer     (HTREEBUFFER);
            int               TranslateIndex    (TVbxTList*, int nFromIndexMethod,
                                                 int nToIndexMethod,
                                                 int nFromIndex);
            int               GetItemByXY       (TVbxTList*, int nX, int nY, int nType);
            int               GetItemRect       (TVbxTList*, int nIndex, int nOpts,
                                                 LPRECT lpRectT);
            int               SaveBuffer        (HTREEBUFFER hTreeBuffer,
                                                 int nFile);
            int               LoadBuffer        (HTREEBUFFER hTreeBuffer,
                                                 int nFile);
            int               CopyBuffer        (HTREEBUFFER hTreeBuffer);
            int               PasteBuffer       (HTREEBUFFER hTreeBuffer);
            int               IsClipboardFormatAvailable
                                                ();
            int               IndexByBM         (TVbxTList*, LONG hTreeBookmark);
                                                int               IsValidBM         (TVbxTList*, LONG hTreeBookmark);
            int               FindItem          (TVbxTList*, LPSTR lpFindWhat, int nFlags,
                                                 int nFromIndex, int nToIndex);
};


TListLoader g_TListLoader;

/////////////////////////////////////////////////////////////////////////////
// TVbxTList

BOOL TVbxTList::AddItem(int nIndex, LPCSTR lpsz)
{
#if __BORLANDC__ < 0x452
  HSZ hsz = ::VBXCreateCString(HANDLE(_DS), LPSTR(lpsz));
  int nRetVal = TVbxControl::AddItem(nIndex, hsz);
        ::VBXDestroyCString(hsz);
  return nRetVal;
#else
  return TVbxControl::AddItem(nIndex, lpsz);
#endif
}

BOOL TVbxTList::Clear()
{
  return ::VBXMethod(GetHCTL(), 0x000A, NULL);
}

BOOL TVbxTList::SetProperty
(
  int propIndex, LONG lData, int arrayIndex, int nVBType
)
{
  ELEMENTSTRUCT elem;
  elem.Value            = lData;
  elem.NumElems         = 1;
  elem.Element[0].Type  = nVBType;
  elem.Element[0].Index = arrayIndex;
        return ::VBXSetProp(GetHCTL(), propIndex, LONG(&elem));
}

BOOL TVbxTList::GetProperty
(
  int propIndex, LONG & lData, int arrayIndex, int nVBType
)
{
  ELEMENTSTRUCT elem;
        elem.Value            = 0;
        elem.NumElems         = 1;
        elem.Element[0].Type  = nVBType;
        elem.Element[0].Index = arrayIndex;
        int nRetVal = ::VBXGetProp(GetHCTL(), propIndex, LPVOID(&elem));
        if(nRetVal)
        lData = elem.Value;
        return nRetVal;
}

BOOL TVbxTList::GetProp(int propIndex, ENUM & Data, int arrayIndex)
{
        LONG lData;
        if(GetProperty(propIndex, lData, arrayIndex, PTYPE_ENUM))
        {
                Data = ENUM(lData);
                return TRUE;
        }
        return FALSE;
}

BOOL TVbxTList::GetProp(int propIndex, int & Data, int arrayIndex)
{
        LONG lData;
        if(GetProperty(propIndex, lData, arrayIndex, PTYPE_SHORT))
        {
                Data = int(lData);
                return TRUE;
        }
        return FALSE;
}

BOOL TVbxTList::GetProp(int propIndex, string & value, int arrayIndex)
{
  LONG lData;
  if(GetProperty(propIndex, lData, arrayIndex, PTYPE_CSTRING))
  {
    LPSTR str = ::VBXGetCStringPtr(HSZ(lData));
                value = str;
                ::VBXDestroyCString(HSZ(lData));
    return TRUE;
  }
  return FALSE;
}

BOOL TVbxTList::GetProp(int propIndex, LONG & value, int arrayIndex)
{
  return GetProperty(propIndex, value, arrayIndex, PTYPE_LONG);
}

BOOL TVbxTList::GetProp(int propIndex, float & value, int arrayIndex)
{
  LONG lData;
  if(GetProperty(propIndex, lData, arrayIndex, PTYPE_REAL))
  {
    value = *(float *)&lData;
    return TRUE;
  }
  return FALSE;
}

BOOL TVbxTList::GetProp(int propIndex, HPIC & value, int arrayIndex)
{
  LONG lData;
  if(GetProperty(propIndex, lData, arrayIndex, PTYPE_PICTURE))
  {
    value = HPIC(lData);
    return TRUE;
  }
  return FALSE;
}

// TList's functions
void TVbxTList::FreeBuffer(HTREEBUFFER hTreeBuffer)
{
  g_TListLoader.FreeBuffer(hTreeBuffer);
}

BOOL TVbxTList::IsValidBuffer(HTREEBUFFER hTreeBuffer)
{
  return g_TListLoader.IsValidBuffer(hTreeBuffer);
}

int TVbxTList::TranslateIndex
(
  int nFromIndexMethod, int nToIndexMethod, int nFromIndex
)
{
  return g_TListLoader.TranslateIndex(this,
                         nFromIndexMethod, nToIndexMethod, nFromIndex);
}

int TVbxTList::GetItemByXY(int nX, int nY, int nType)
{
  return g_TListLoader.GetItemByXY(this, nX, nY, nType);
}

int TVbxTList::GetItemRect(int nIndex, int nOpts, LPRECT lpRectT)
{
  return g_TListLoader.GetItemRect(this, nIndex, nOpts, lpRectT);
}

int TVbxTList::SaveBuffer(HTREEBUFFER hTreeBuffer, int nFile)
{
  return g_TListLoader.SaveBuffer(hTreeBuffer, nFile);
}

int TVbxTList::LoadBuffer(HTREEBUFFER hTreeBuffer, int nFile)
{
  return g_TListLoader.LoadBuffer(hTreeBuffer, nFile);
}

int TVbxTList::CopyBuffer(HTREEBUFFER hTreeBuffer)
{
  return g_TListLoader.CopyBuffer(hTreeBuffer);
}

int TVbxTList::PasteBuffer(HTREEBUFFER hTreeBuffer)
{
  return g_TListLoader.PasteBuffer(hTreeBuffer);
}

int TVbxTList::IsClipboardFormatAvailable()
{
  return g_TListLoader.IsClipboardFormatAvailable();
}

int TVbxTList::IndexByBM(LONG hTreeBookmark)
{
  return g_TListLoader.IndexByBM(this, hTreeBookmark);
}

int TVbxTList::IsValidBM(LONG hTreeBookmark)
{
  return g_TListLoader.IsValidBM(this, hTreeBookmark);
}

int TVbxTList::FindItem
(
  LPSTR lpFindWhat, int nFlags, int nFromIndex, int nToIndex
)
{
  return g_TListLoader.FindItem(this,
                         lpFindWhat, nFlags, nFromIndex, nToIndex);
}

/////////////////////////////////////////////////////////////////////////////
// TVbxTList message handlers

TListLoader::TListLoader()
{
  m_hmod = LoadLibrary("TLISTPRO.VBX");

  if(m_hmod >= HINSTANCE_ERROR)
  {
    m_fpFreeBuffer     = GetProcAddress(m_hmod, "TListFreeBuffer");
    m_fpIsValidBuffer  = GetProcAddress(m_hmod, "TListIsValidBuffer");
    m_fpTranslateIndex = GetProcAddress(m_hmod, "TListTranslateIndex");
    m_fpGetItemByXY    = GetProcAddress(m_hmod, "TListGetItemByXY");
    m_fpGetItemRect    = GetProcAddress(m_hmod, "TListGetItemRect");
    m_fpSaveBuffer     = GetProcAddress(m_hmod, "TListSaveBuffer");
    m_fpLoadBuffer     = GetProcAddress(m_hmod, "TListLoadBuffer");
    m_fpCopyBuffer     = GetProcAddress(m_hmod, "TListCopyBuffer");
    m_fpPasteBuffer    = GetProcAddress(m_hmod, "TListPasteBuffer");
    m_fpIsClipboardFormatAvailable =
                         GetProcAddress(m_hmod,
                           "TListIsClipboardFormatAvailable");
    m_fpIndexByBM      = GetProcAddress(m_hmod, "TListIndexByBM");
    m_fpIsValidBM      = GetProcAddress(m_hmod, "TListIsValidBM");
    m_fpFindItem       = GetProcAddress(m_hmod, "TListFindItem");
  }
  else
  {
    m_fpFreeBuffer    = NULL;
    m_fpIsValidBuffer = NULL;
    m_hmod = NULL;
                TRACE("Couldn't load TLISTPRO.VBX\n");
  }
}

TListLoader::~TListLoader()
{
  if(m_hmod)
    FreeLibrary(m_hmod);
}

void TListLoader::FreeBuffer(HTREEBUFFER hTreeBuffer)
{
  if(m_fpFreeBuffer)
    (*(void WINAPI (*)(HTREEBUFFER))m_fpFreeBuffer)(hTreeBuffer);
}

BOOL TListLoader::IsValidBuffer(HTREEBUFFER hTreeBuffer)
{
  return m_fpIsValidBuffer
         ? (*(BOOL WINAPI (*)(HTREEBUFFER))m_fpIsValidBuffer)(hTreeBuffer)
         : FALSE;
}

int TListLoader::TranslateIndex
(
  TVbxTList *pListTree, int nFromIndexMethod, int nToIndexMethod,
  int nFromIndex
)
{
  return m_fpTranslateIndex ?
    (*(int WINAPI (*)(HCTL, int, int, int))m_fpTranslateIndex)
      (pListTree->GetHCTL(), nFromIndexMethod, nToIndexMethod, nFromIndex) :
     FALSE;
}

int TListLoader::GetItemByXY(TVbxTList *pListTree, int nX, int nY, int nType)
{
  return m_fpGetItemByXY ?
    (*(int WINAPI (*)(HCTL, int, int, int))m_fpGetItemByXY)
    (pListTree->GetHCTL(), nX, nY, nType) :
    FALSE;
}

int TListLoader::GetItemRect(TVbxTList *pListTree, int nIndex, int nOpts,
                                                                                                                        LPRECT lpRectT)
{
  return m_fpGetItemRect ?
    (*(int WINAPI (*)(HCTL, int, int, LPRECT))m_fpGetItemRect)
    (pListTree->GetHCTL(), nIndex, nOpts, lpRectT) :
    FALSE;
}

int TListLoader::SaveBuffer(HTREEBUFFER hTreeBuffer, int nFile)
{
  return m_fpSaveBuffer ?
    (*(int WINAPI (*)(HTREEBUFFER, int))m_fpSaveBuffer)
    (hTreeBuffer, nFile) : FALSE;
}

int TListLoader::LoadBuffer(HTREEBUFFER hTreeBuffer, int nFile)
{
  return m_fpLoadBuffer ?
  (*(int WINAPI (*)(HTREEBUFFER, int))m_fpLoadBuffer)
  (hTreeBuffer, nFile) : FALSE;
}

int TListLoader::CopyBuffer(HTREEBUFFER hTreeBuffer)
{
  return m_fpCopyBuffer ?
  (*(int WINAPI (*)(HTREEBUFFER))m_fpCopyBuffer)
  (hTreeBuffer) : FALSE;
}

int TListLoader::PasteBuffer(HTREEBUFFER hTreeBuffer)
{
  return m_fpPasteBuffer ?
  (*(int WINAPI (*)(HTREEBUFFER))m_fpPasteBuffer)
  (hTreeBuffer) : FALSE;
}

int TListLoader::IsClipboardFormatAvailable()
{
  return m_fpIsClipboardFormatAvailable ?
  (*(int WINAPI (*)())m_fpIsClipboardFormatAvailable)() : FALSE;
}

int TListLoader::IndexByBM(TVbxTList *pListTree, LONG hTreeBookmark)
{
  return m_fpIndexByBM ?
  (*(int WINAPI (*)(HCTL, LONG))m_fpIndexByBM)
  (pListTree->GetHCTL(), hTreeBookmark) : FALSE;
}

int TListLoader::IsValidBM(TVbxTList *pListTree, LONG hTreeBookmark)
{
  return m_fpIsValidBM ?
  (*(int WINAPI (*)(HCTL, LONG))m_fpIsValidBM)
  (pListTree->GetHCTL(), hTreeBookmark) : FALSE;
}

int TListLoader::FindItem(TVbxTList *pListTree, LPSTR lpFindWhat,
                                                                                                         int nFlags, int nFromIndex, int nToIndex)
{
  return m_fpFindItem ?
  (*(int WINAPI (*)(HCTL, LPSTR, int, int, int))m_fpFindItem)
  (pListTree->GetHCTL(), lpFindWhat, nFlags, nFromIndex, nToIndex)
  : FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// TList's functions

VOID WINAPI TListFreeBuffer(HTREEBUFFER hTreeBuffer)
{
  g_TListLoader.FreeBuffer(hTreeBuffer);
}

int WINAPI TListIsValidBuffer(HTREEBUFFER hTreeBuffer)
{
  return g_TListLoader.IsValidBuffer(hTreeBuffer);
}

int WINAPI TListTranslateIndex(TVbxTList* pListTree, int nFromIndexMethod,
                               int nToIndexMethod, int nFromIndex)
{
  return g_TListLoader.TranslateIndex(pListTree, nFromIndexMethod,
                                      nToIndexMethod, nFromIndex);
}

int WINAPI TListGetItemByXY(TVbxTList* pListTree, int nX, int nY, int nType)
{
  return g_TListLoader.GetItemByXY(pListTree, nX, nY, nType);
}

int WINAPI TListGetItemRect(TVbxTList* pListTree, int nIndex, int nOpts,
                            LPRECT lpRectT)
{
  return g_TListLoader.GetItemRect(pListTree, nIndex, nOpts, lpRectT);
}

int WINAPI TListSaveBuffer(HTREEBUFFER hTreeBuffer, int nFile)
{
  return g_TListLoader.SaveBuffer(hTreeBuffer, nFile);
}

int WINAPI TListLoadBuffer(HTREEBUFFER hTreeBuffer, int nFile)
{
  return g_TListLoader.LoadBuffer(hTreeBuffer, nFile);
}

int WINAPI TListCopyBuffer(HTREEBUFFER hTreeBuffer)
{
  return g_TListLoader.CopyBuffer(hTreeBuffer);
}

int WINAPI TListPasteBuffer(HTREEBUFFER hTreeBuffer)
{
  return g_TListLoader.PasteBuffer(hTreeBuffer);
}

int WINAPI TListIsClipboardFormatAvailable()
{
  return g_TListLoader.IsClipboardFormatAvailable();
}

int WINAPI TListIndexByBM(TVbxTList* pListTree, LONG hTreeBookmark)
{
  return g_TListLoader.IndexByBM(pListTree, hTreeBookmark);
}

int WINAPI TListIsValidBM(TVbxTList* pListTree, LONG hTreeBookmark)
{
  return g_TListLoader.IsValidBM(pListTree, hTreeBookmark);
}

int WINAPI TListFindItem(TVbxTList* pListTree, LPSTR lpFindWhat,
                         int nFlags, int nFromIndex, int nToIndex)
{
  return g_TListLoader.FindItem(pListTree, lpFindWhat, nFlags,
                                nFromIndex, nToIndex);
}

/** (END OF FILE  : tl30p_bc.cpp) *******************************************/
